#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 1 * VramStep, "SCORE");
    // PrintS(Vram + 7 * VramStep, "HI-SCORE");
    PrintS(Vram + 7 * VramStep, "ST");
    PrintByteNumber2(Vram + VramRowSize + 7 * VramStep, CurrentStage + 1);
    PrintS(Vram+ 10 * VramStep, "TIME");
    {
        static const byte[] RemainChars = {
            Char_MyFort + 2, 
            Char_MyFort + 3,
            Char_MyFort + 6,
            Char_MyFort + 7,
        };
        ptr<byte> pVram;
        pVram = Vram + 15 * VramStep;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                ptr<byte> pChars;
                pChars = RemainChars;
                repeat (2) {
                    repeat (2) {
                        pVram = Put(pVram, *pChars);
                        ++pChars;
                    }
                    pVram += VramRowSize - 2 * VramStep;
                }
                pVram += 2 * VramStep - VramRowSize * 2;
                --i;
            } while (i > 0);
        }
    }
    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    pVram = PrintNumber5(Vram + VramRowSize, Score);
    PrintC(pVram, '0');
    // pVram = PrintNumber5(Vram + VramRowSize + 9 * VramStep, HiScore);
    // PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize + 11 * VramStep, StageTime);
}


void PrintGameOver()
{
    PrintS(Vram + VramRowSize * 7 + (WindowWidth - 9) / 2 * VramStep, "GAME OVER");
}

void PrintTimeUp() 
{
    PrintS(Vram + VramRowSize * 7 + (WindowWidth - 7) / 2 * VramStep, "TIME UP");
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    HScroll = 0;
    PrintStatus();
    {
        static const byte[] TitleBytes = {
            0x2c, 0x22, 0x28, 0x1d, 
            0x2c, 0x27, 0x24, 0x29, 
            0x2c, 0x1d, 0x2c, 0x29, 
            0x22, 0x22, 0x1e, 0x21, 
            0x2b, 0x28, 0x1d, 0x2a, 
            0x20, 0x29, 0x20, 0x29, 
            0x24, 0x2a, 0x20, 0x29, 
            0x1e, 0x21, 0x1e, 0x21, 
            0x24, 0x21, 0x2c, 0x1e, 
            0x20, 0x1d, 0x2c, 0x1d, 
            0x20, 0x1d, 0x2c, 0x1d, 
            0x1e, 0x1d, 0x22, 0x1d, 
            0x2c, 0x1d, 0x25, 0x24, 
            0x2c, 0x1d, 0x29, 0x20, 
            0x2c, 0x1d, 0x29, 0x20, 
            0x22, 0x22, 0x1d, 0x22, 
            0x2a, 0x1f, 0x2a, 0x24, 
            0x29, 0x20, 0x29, 0x20, 
            0x29, 0x20, 0x29, 0x20, 
            0x22, 0x1d, 0x21, 0x1e, 
        };
        constexpr byte LogoLength = 5;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 4 + (WindowWidth - 4 * LogoLength) / 2 * VramStep;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 4;
        }
        PrintS(Vram + VramRowSize * 8 + ((WindowWidth + 4 * LogoLength) / 2 - 4) * VramStep, "MINI");
    }
    PrintS(Vram + VramRowSize * 13 + 1 * VramStep, "PUSH START BUTTON");
    PrintS(Vram + VramRowSize * 15 + 8 * VramStep, "INUFUTO 2024");
}